const severeWarningEvents = ["Severe Thunderstorm Warning", "Destructive Severe Thunderstorm Warning", "Considerable Destructive Severe Thunderstorm Warning"]
const tornadoWarningEvents = ["Tornado Warning", "Radar Indicated Tornado Warning", "Confirmed Tornado Warning", "Tornado Emergency"]

let alerts = msg.payload

// Helper function to safely access nested properties
const getNestedProperty = (obj, path, defaultValue = undefined) => {
    return path.reduce((acc, key) => (acc && acc[key] !== undefined ? acc[key] : defaultValue), obj)
}

// Filter function
const filterAlerts = (alerts, condition) => {
    return alerts.filter(alert => {
        try {
            return condition(alert)
        } catch (e) {
            node.warn(`Error processing alert: ${e.message}`)
            return false
        }
    })
}

// Filtering logic
const ts = filterAlerts(alerts, alert => {
    const event = getNestedProperty(alert, ['raw', 'properties', 'event'])
    return severeWarningEvents.includes(event)
})

const tstp = filterAlerts(alerts, alert => {
    const event = getNestedProperty(alert, ['raw', 'properties', 'event'])
    const tornadoDetection = getNestedProperty(alert, ['raw', 'properties', 'parameters', 'tornadoDetection'], [])
    return severeWarningEvents.includes(event) && tornadoDetection.includes("POSSIBLE")
})

const cdst = filterAlerts(alerts, alert => {
    const event = getNestedProperty(alert, ['raw', 'properties', 'event'])
    return event === "Considerable Destructive Severe Thunderstorm Warning"
})

const tornado = filterAlerts(alerts, alert => {
    const event = getNestedProperty(alert, ['raw', 'properties', 'event'])
    return tornadoWarningEvents.includes(event)
})

const confirmed_tornado = filterAlerts(alerts, alert => {
    const event = getNestedProperty(alert, ['raw', 'properties', 'event'])
    return event === "Confirmed Tornado Warning"
})

// Output messages
let tstormMsg = {
    "payload": {
        "alerts": ts,
        "count": ts.length,
        "tornado_possible": tstp.length,
        "considerable_destructive": cdst.length
    }
}

let tornadoWarnMsg = {
    "payload": {
        "alerts": tornado,
        "count": tornado.length,
        "confirmed": confirmed_tornado.length
    }
}

node.send([tstormMsg, tornadoWarnMsg])
node.status({ fill: 'green', shape: 'dot', text: `${alerts.length} alerts processed at ${new Date().toLocaleString()}`})